@@ -23,7 +23,7 @@
 #include "katemdi.h"
 #include "katemdi.moc"
 
-#include "kate/pluginconfigpageinterface.h"
+#include "interfaces/kate/pluginconfigpageinterface.h"
 
 #include <kactioncollection.h>
 #include <kactionmenu.h>
@@ -307,7 +307,7 @@ namespace KateMDI
       widget->m_sidebar = this;
     }
 
-    // save its pos ;)
+    // save it's pos ;)
     widget->persistent = false;
 
     m_idToWidget.insert (newId, widget);
@@ -378,10 +378,6 @@ namespace KateMDI
 
     setTab (m_widgetToId[widget], true);
 
-    // set minimum size again to 80,80!
-    // we changed that on hide!
-    widget->setMinimumSize (80, 80);
-    
     /**
      * resize to right size again and show, else artefacts
      */
@@ -437,10 +433,6 @@ namespace KateMDI
         m_preHideSize = m_ownSplit->size ();
       m_ownSplit->hide ();
     }
-    
-    // set minimum size == size, this avoid artifical resizes on show
-    // there we will reset this again to 80,80!
-    widget->setMinimumSize (widget->size());
 
     widget->hide ();
 
