@@ -40,7 +40,6 @@
 // on the fly compression
 #include <kfilterdev.h>
 #include <kmimetype.h>
-#include <kde_file.h>
 
 #include <QtCore/QFile>
 #include <QtCore/QTextStream>
@@ -50,6 +49,13 @@
 
 #include <limits.h>
 
+/**
+ * hl will look at the next KATE_HL_LOOKAHEAD lines
+ * or until the current block ends if a line is requested
+ * will avoid to run doHighlight too often
+ */
+static const int KATE_HL_LOOKAHEAD = 64;
+
 /**
  * Initial value for m_maxDynamicContexts
  */
@@ -64,6 +70,7 @@ KateBuffer::KateBuffer(KateDocument *doc)
    m_brokenEncoding (false),
    m_tooLongLinesWrapped (false),
    m_highlight (0),
+   m_regionTree (this),
    m_tabWidth (8),
    m_lineHighlighted (0),
    m_maxDynamicContexts (KATE_MAX_DYNAMIC_CONTEXTS)
@@ -93,56 +100,28 @@ void KateBuffer::editStart ()
 
 void KateBuffer::editEnd ()
 {
-  /**
-   * not finished, do nothing
-   */
   if (!finishEditing())
     return;
 
-  /**
-   * nothing change, OK
-   */
-  if (!editingChangedBuffer ())
-    return;
+  if (editingChangedBuffer ())
+  {
+    // hl update !!!
+    if (m_highlight && editingMinimalLineChanged () <= editingMaximalLineChanged () && editingMaximalLineChanged () <= m_lineHighlighted)
+    {
+      // look one line too far, needed for linecontinue stuff
+      int editTagLineEnd = editingMaximalLineChanged () + 1;
+      int editTagLineStart = editingMinimalLineChanged ();
 
-  /**
-   * if we arrive here, line changed should be OK
-   */
-  Q_ASSERT (editingMinimalLineChanged () != -1);
-  Q_ASSERT (editingMaximalLineChanged () != -1);
-  Q_ASSERT (editingMinimalLineChanged () <= editingMaximalLineChanged ());
-  
-  /**
-   * no highlighting, nothing to do
-   */
-  if (!m_highlight)
-    return;
+      // look one line before, needed nearly 100% only for indentation based folding !
+      if (editTagLineStart > 0)
+        --editTagLineStart;
 
-  /**
-   * if we don't touch the highlighted area => fine
-   */
-  if (editingMinimalLineChanged() > m_lineHighlighted)
-    return;
-
-  /**
-   * look one line too far, needed for linecontinue stuff
-   */
-  int editTagLineEnd = editingMaximalLineChanged () + 1;
-  int editTagLineStart = editingMinimalLineChanged ();
-
-  /**
-   * look one line before, needed nearly 100% only for indentation based folding !
-   */
-  if (editTagLineStart > 0)
-    --editTagLineStart;
-
-  /**
-   * really update highlighting
-   */
-  doHighlight (
-      editTagLineStart,
-      editTagLineEnd,
-      true);
+      doHighlight (
+          editTagLineStart,
+          editTagLineEnd,
+          true);
+    }
+  }
 }
 
 void KateBuffer::clear()
@@ -150,6 +129,8 @@ void KateBuffer::clear()
   // call original clear function
   Kate::TextBuffer::clear ();
 
+  m_regionTree.clear();
+
   // reset the state
   m_brokenEncoding = false;
   m_tooLongLinesWrapped = false;
@@ -158,7 +139,7 @@ void KateBuffer::clear()
   m_lineHighlighted = 0;
 }
 
-bool KateBuffer::openFile (const QString &m_file, bool enforceTextCodec)
+bool KateBuffer::openFile (const QString &m_file)
 {
   // first: setup fallback and normal encoding
   setEncodingProberType (KateGlobalConfig::global()->proberType ());
@@ -168,8 +149,10 @@ bool KateBuffer::openFile (const QString &m_file, bool enforceTextCodec)
   // setup eol
   setEndOfLineMode ((EndOfLineMode) m_doc->config()->eol());
 
-  // NOTE: we do not remove trailing spaces on load. This was discussed
-  //       over the years again and again. bugs: 306926, 239077, ...
+  // remove trailing spaces?
+  // NOTE: The buffer won't actually remove trailing space on load. This is because
+  // we need to do it later, after the config and variables have been parsed.
+  setRemoveTrailingSpaces (m_doc->config()->removeSpaces());
 
   // line length limit
   setLineLengthLimit (m_doc->config()->lineLengthLimit());
@@ -177,41 +160,7 @@ bool KateBuffer::openFile (const QString &m_file, bool enforceTextCodec)
   // then, try to load the file
   m_brokenEncoding = false;
   m_tooLongLinesWrapped = false;
-
-  /**
-   * allow non-existent files without error, if local file!
-   * will allow to do "kate newfile.txt" without error messages but still fail if e.g. you mistype a url
-   * and it can't be fetched via fish:// or other strange things in kio happen...
-   * just clear() + exit with success!
-   */  
-  if (m_doc->url().isLocalFile() && !QFile::exists (m_file)) {
-    clear ();
-    KTextEditor::Message* message = new KTextEditor::Message(i18nc("short translation, user created new file", "New file"),
-                                                             KTextEditor::Message::Warning);
-    message->setPosition(KTextEditor::Message::TopInView);
-    message->setAutoHide(1000);
-    m_doc->postMessage(message);
-    
-    // remember error
-    m_doc->setOpeningError(true);
-    m_doc->setOpeningErrorMessage(i18n ("The file %1 does not exist.", m_doc->url().pathOrUrl()));
-    return true;
-  }
-  
-  /**
-   * check if this is a normal file or not, avoids to open char devices or directories!
-   * else clear buffer and break out with error
-   */
-  KDE_struct_stat sbuf;
-  if (KDE::stat(m_file, &sbuf) != 0 || !S_ISREG(sbuf.st_mode)) {
-    clear ();
-    return false;
-  }
-  
-  /**
-   * try to load
-   */
-  if (!load (m_file, m_brokenEncoding, m_tooLongLinesWrapped, enforceTextCodec))
+  if (!load (m_file, m_brokenEncoding, m_tooLongLinesWrapped))
     return false;
 
   // save back encoding
@@ -225,6 +174,9 @@ bool KateBuffer::openFile (const QString &m_file, bool enforceTextCodec)
   if (generateByteOrderMark())
     m_doc->config()->setBom (true);
 
+  // fix region tree
+  m_regionTree.fixRoot (lines ());
+
   // okay, loading did work
   return true;
 }
@@ -266,8 +218,8 @@ bool KateBuffer::saveFile (const QString &m_file)
   // generate bom?
   setGenerateByteOrderMark (m_doc->config()->bom());
 
-  // append a newline character at the end of the file (eof) ?
-  setNewLineAtEof (m_doc->config()->newLineAtEof());
+  // remove trailing spaces?
+  setRemoveTrailingSpaces (m_doc->config()->removeSpaces());
 
   // try to save
   if (!save (m_file))
@@ -281,7 +233,7 @@ bool KateBuffer::saveFile (const QString &m_file)
   return true;
 }
 
-void KateBuffer::ensureHighlighted (int line, int lookAhead)
+void KateBuffer::ensureHighlighted (int line)
 {
   // valid line at all?
   if (line < 0 || line >= lines ())
@@ -291,8 +243,8 @@ void KateBuffer::ensureHighlighted (int line, int lookAhead)
   if (line < m_lineHighlighted)
     return;
 
-  // update hl until this line + max lookAhead
-  int end = qMin(line + lookAhead, lines ()-1);
+  // update hl until this line + max KATE_HL_LOOKAHEAD
+  int end = qMin(line + KATE_HL_LOOKAHEAD, lines ()-1);
 
   // ensure we have enough highlighted
   doHighlight ( m_lineHighlighted, end, false );
@@ -305,6 +257,9 @@ void KateBuffer::wrapLine (const KTextEditor::Cursor &position)
 
   if (m_lineHighlighted > position.line()+1)
     m_lineHighlighted++;
+
+  m_regionTree.lineHasBeenInserted (position.line(), position.column());
+
 }
 
 void KateBuffer::unwrapLines (int from, int to)
@@ -314,24 +269,21 @@ void KateBuffer::unwrapLines (int from, int to)
   Q_ASSERT(to + 1 <= lines());
 
   for (int line = to; line >= from; --line) {
-      if (line + 1 < lines()) {
+      if (line + 1 < lines())
           Kate::TextBuffer::unwrapLine (line + 1);
-        
-          if (m_lineHighlighted > (line + 1))
-            --m_lineHighlighted;
-      }
 
       // Line "0" can't be unwraped
       // This call is used to unwrap the last line (if last line != 0)
       // This call was used in the previous version too and it looks like the last
       // line can't be unwraped without it
-      else if (line) {
+      else if (line)
           Kate::TextBuffer::unwrapLine (line);
-        
-          if (m_lineHighlighted > line)
-            --m_lineHighlighted;
-      }
   }
+
+  if (m_lineHighlighted > from)
+    m_lineHighlighted = from;
+
+  m_regionTree.linesHaveBeenRemoved (from, to);
 }
 
 void KateBuffer::unwrapLine (int line)
@@ -341,6 +293,8 @@ void KateBuffer::unwrapLine (int line)
 
   if (m_lineHighlighted > line)
     --m_lineHighlighted;
+
+  m_regionTree.linesHaveBeenRemoved (line, line);
 }
 
 void KateBuffer::setTabWidth (int w)
@@ -371,6 +325,10 @@ void KateBuffer::setHighlight(int hlMode)
 
     h->use();
 
+    // Clear code folding tree (see bug #124102)
+    m_regionTree.clear();
+    m_regionTree.fixRoot(lines());
+
     m_highlight = h;
 
     if (invalidate)
@@ -391,10 +349,75 @@ void KateBuffer::invalidateHighlighting()
   m_lineHighlighted = 0;
 }
 
+void KateBuffer::updatePreviousNotEmptyLine(int current_line,bool addindent,int deindent)
+{
+  Kate::TextLine textLine;
+  do {
+    if (current_line == 0) return;
+
+    --current_line;
+
+    textLine = plainLine (current_line);
+  } while (textLine->firstChar()==-1);
+
+  kDebug(13020)<<"updatePreviousNotEmptyLine: updating line:"<<current_line;
+  QVector<int> foldingList=textLine->foldingListArray();
+  while ( !foldingList.isEmpty() && abs(foldingList.at(foldingList.size()-2)) == 1) {
+    foldingList.resize(foldingList.size()-2);
+  }
+  addIndentBasedFoldingInformation(foldingList,textLine->length(),addindent,deindent);
+  textLine->setFoldingList(foldingList);
+
+  bool retVal_folding = false;
+  m_regionTree.updateLine(current_line, foldingList, &retVal_folding, true,false);
+
+  // tagLines() is emitted from KatBuffer::doHighlight()!
+}
+
+void KateBuffer::addIndentBasedFoldingInformation(QVector<int> &foldingList,int linelength,bool addindent,int deindent)
+{
+  if (addindent) {
+    //kDebug(13020)<<"adding indent for line :"<<current_line + buf->startLine()<<"  textLine->noIndentBasedFoldingAtStart"<<textLine->noIndentBasedFoldingAtStart();
+    //kDebug(13020)<<"adding ident";
+    foldingList.resize (foldingList.size() + 2);
+    foldingList[foldingList.size()-2] = 1;
+    foldingList[foldingList.size()-1] = 0;
+  }
+  //kDebug(13020)<<"DEINDENT: "<<deindent;
+  if (deindent > 0)
+  {
+    //foldingList.resize (foldingList.size() + (deindent*2));
+
+    //Make the whole last line marked as still belonging to the block
+    for (int z=0;z<deindent;z++) {
+      foldingList << -1 << linelength+1;
+    }
+
+/*    for (int z= foldingList.size()-(deindent*2); z < foldingList.size(); z=z+2)
+    {
+      foldingList[z] = -1;
+      foldingList[z+1] = 0;
+    }*/
+  }
+}
+
+
+bool KateBuffer::isEmptyLine(Kate::TextLine textline)
+{
+  QLinkedList<QRegExp> l;
+  l=m_highlight->emptyLines(textline->attribute(0));
+  if (l.isEmpty()) return false;
+  QString txt=textline->string();
+  foreach(const QRegExp &re,l) {
+    if (re.exactMatch(txt)) return true;
+  }
+  return false;
+}
+
 void KateBuffer::doHighlight (int startLine, int endLine, bool invalidate)
 {
   // no hl around, no stuff to do
-  if (!m_highlight || m_highlight->noHighlighting())
+  if (!m_highlight)
     return;
 
 #ifdef BUFFER_DEBUGGING
@@ -440,28 +463,35 @@ void KateBuffer::doHighlight (int startLine, int endLine, bool invalidate)
     }
   }
 
-  // if possible get previous line, otherwise create 0 line.
-  Kate::TextLine prevLine = (startLine >= 1) ? plainLine (startLine - 1) : Kate::TextLine ();
+  // get previous line, if any
+  Kate::TextLine prevLine;
+
+  if (startLine >= 1)
+    prevLine = plainLine (startLine-1);
+  else
+    prevLine = Kate::TextLine (new Kate::TextLineData ());
+
+  // does we need to emit a signal for the folding changes ?
+  bool codeFoldingUpdate = false;
 
   // here we are atm, start at start line in the block
   int current_line = startLine;
   int start_spellchecking = -1;
   int last_line_spellchecking = -1;
+  bool indentContinueWhitespace=false;
+  bool indentContinueNextWhitespace=false;
   bool ctxChanged = false;
-  Kate::TextLine textLine = plainLine (current_line);
-  Kate::TextLine nextLine;
   // loop over the lines of the block, from startline to endline or end of block
   // if stillcontinue forces us to do so
   for (; current_line < qMin (endLine+1, lines()); ++current_line)
   {
-    // get next line, if any    
-    if ((current_line + 1) < lines())
-      nextLine = plainLine (current_line+1);
-    else
-      nextLine = Kate::TextLine (new Kate::TextLineData ());
-    
+    // current line
+    Kate::TextLine textLine = plainLine (current_line);
+
+    QVector<int> foldingList;
     ctxChanged = false;
-    m_highlight->doHighlight (prevLine.data(), textLine.data(), nextLine.data(), ctxChanged, tabWidth());
+
+    m_highlight->doHighlight (prevLine.data(), textLine.data(), foldingList, ctxChanged);
 
 #ifdef BUFFER_DEBUGGING
     // debug stuff
@@ -475,9 +505,210 @@ void KateBuffer::doHighlight (int startLine, int endLine, bool invalidate)
     }
     kDebug( 13020 );
 #endif
-    
+
+    //
+    // indentation sensitive folding
+    //
+    bool indentChanged = false;
+    if (m_highlight->foldingIndentationSensitive())
+    {
+      // get the indentation array of the previous line to start with !
+      QVector<unsigned short> indentDepth (prevLine->indentationDepthArray());
+
+      // current indentation of this line
+      int iDepth = textLine->indentDepth(m_tabWidth);
+      if (current_line==0)
+      {
+          indentDepth.resize (1);
+          indentDepth[0] = iDepth;
+      }
+
+      textLine->setNoIndentBasedFoldingAtStart(prevLine->noIndentBasedFolding());
+
+      // this line is empty, beside spaces, or has indentaion based folding disabled, use indentation depth of the previous line !
+
+#ifdef BUFFER_DEBUGGING
+      kDebug(13020)<<"current_line:"<<current_line<<" textLine->noIndentBasedFoldingAtStart"<<textLine->noIndentBasedFoldingAtStart();
+#endif
+
+      if ( (textLine->firstChar() == -1) || textLine->noIndentBasedFoldingAtStart() || isEmptyLine(textLine) )
+      {
+        // do this to get skipped empty lines indent right, which was given in the indenation array
+        if (!prevLine->indentationDepthArray().isEmpty())
+        {
+          iDepth = prevLine->indentationDepthArray().last();
+
+#ifdef BUFFER_DEBUGGING
+          kDebug(13020)<<"reusing old depth as current";
+#endif
+        }
+        else
+        {
+          iDepth = prevLine->indentDepth(m_tabWidth);
+
+#ifdef BUFFER_DEBUGGING
+          kDebug(13020)<<"creating indentdepth for previous line";
+#endif
+        }
+      }
+
+#ifdef BUFFER_DEBUGGING
+      kDebug(13020)<<"iDepth:"<<iDepth;
+#endif
+
+      // query the next line indentation, if we are at the end of the block
+      // use the first line of the next buf block
+      int nextLineIndentation = 0;
+      bool nextLineIndentationValid=true;
+      indentContinueNextWhitespace=false;
+      if ((current_line+1) < lines())
+      {
+        if ( (plainLine (current_line+1)->firstChar() == -1) || isEmptyLine(plainLine (current_line+1)) )
+        {
+          nextLineIndentation = iDepth;
+          indentContinueNextWhitespace=true;
+        }
+        else
+          nextLineIndentation = plainLine (current_line+1)->indentDepth(m_tabWidth);
+      }
+      else
+      {
+        nextLineIndentationValid=false;
+      }
+
+      if  (!textLine->noIndentBasedFoldingAtStart()) {
+
+        if ((iDepth > 0) && (indentDepth.isEmpty() || (indentDepth.last() < iDepth)))
+        {
+#ifdef BUFFER_DEBUGGING
+          kDebug(13020)<<"adding depth to \"stack\":"<<iDepth;
+#endif
+
+          indentDepth.append (iDepth);
+        } else {
+          if (!indentDepth.isEmpty())
+          {
+            for (int z=indentDepth.size()-1; z > -1; z--)
+              if (indentDepth.at(z) > iDepth)
+                indentDepth.resize(z);
+            if ((iDepth > 0) && (indentDepth.isEmpty() || (indentDepth.last() < iDepth)))
+            {
+#ifdef BUFFER_DEBUGGING
+              kDebug(13020)<<"adding depth to \"stack\":"<<iDepth;
+#endif
+
+              indentDepth.append (iDepth);
+              if (prevLine->firstChar()==-1) {
+
+              }
+            }
+          }
+        }
+      }
+
+      if (!textLine->noIndentBasedFolding())
+      {
+        if (nextLineIndentationValid)
+        {
+          //if (textLine->firstChar()!=-1)
+          {
+#ifdef BUFFER_DEBUGGING
+            kDebug(13020)<<"nextLineIndentation:"<<nextLineIndentation;
+#endif
+
+            bool addindent=false;
+            int deindent=0;
+
+#ifdef BUFFER_DEBUGGING
+            if (!indentDepth.isEmpty())
+              kDebug(13020)<<"indentDepth.last():"<<indentDepth.last();
+#endif
+
+            if (nextLineIndentation > 0 && ( indentDepth.isEmpty() || indentDepth.last() < nextLineIndentation))
+            {
+#ifdef BUFFER_DEBUGGING
+              kDebug(13020)<<"addindent==true";
+#endif
+
+              addindent=true;
+            } else {
+            if (!indentDepth.isEmpty() && indentDepth.last() > nextLineIndentation)
+              {
+#ifdef BUFFER_DEBUGGING
+                kDebug(13020)<<"....";
+#endif
+
+                for (int z=indentDepth.size()-1; z > -1; z--)
+                {
+#ifdef BUFFER_DEBUGGING
+                  kDebug(13020)<<indentDepth.at(z)<<"  "<<nextLineIndentation;
+#endif
+
+                  if (indentDepth.at(z) > nextLineIndentation)
+                    deindent++;
+                }
+              }
+            }
+/*        }
+        if (textLine->noIndentBasedFolding()) kDebug(13020)<<"=============================indentation based folding disabled======================";
+        if (!textLine->noIndentBasedFolding()) {*/
+            if ((textLine->firstChar()==-1)) {
+              updatePreviousNotEmptyLine(current_line,addindent,deindent);
+              codeFoldingUpdate=true;
+            }
+            else
+            {
+              addIndentBasedFoldingInformation(foldingList,textLine->length(),addindent,deindent);
+            }
+          }
+        }
+      }
+      indentChanged = !(indentDepth == textLine->indentationDepthArray());
+
+      // assign the new array to the textline !
+      if (indentChanged)
+        textLine->setIndentationDepth (indentDepth);
+
+      indentContinueWhitespace=textLine->firstChar()==-1;
+    }
+    bool foldingColChanged=false;
+    bool foldingChanged = false; //!(foldingList == textLine->foldingListArray());
+    if (foldingList.size()!=textLine->foldingListArray().size()) {
+      foldingChanged=true;
+    } else {
+      QVector<int>::ConstIterator it=foldingList.constBegin();
+      QVector<int>::ConstIterator it1=textLine->foldingListArray().constBegin();
+      bool markerType=true;
+      for(;it!=foldingList.constEnd();++it,++it1) {
+        if  (markerType) {
+          if ( ((*it)!=(*it1))) {
+            foldingChanged=true;
+            foldingColChanged=false;
+            break;
+          }
+        } else {
+            if ((*it)!=(*it1)) {
+              foldingColChanged=true;
+            }
+        }
+        markerType=!markerType;
+      }
+    }
+
+    if (foldingChanged || foldingColChanged) {
+      textLine->setFoldingList(foldingList);
+      if (foldingChanged==false){
+        textLine->setFoldingColumnsOutdated(textLine->foldingColumnsOutdated() | foldingColChanged);
+      } else textLine->setFoldingColumnsOutdated(false);
+    }
+    bool retVal_folding = false;
+    //perhaps make en enums out of the change flags
+    m_regionTree.updateLine(current_line, foldingList, &retVal_folding, foldingChanged,foldingColChanged);
+
+    codeFoldingUpdate = codeFoldingUpdate | retVal_folding;
+
     // need we to continue ?
-    bool stillcontinue =  ctxChanged;
+    bool stillcontinue =  ctxChanged || indentChanged || indentContinueWhitespace || indentContinueNextWhitespace;
     if (stillcontinue && start_spellchecking < 0) {
       start_spellchecking=current_line;
     }
@@ -487,7 +718,6 @@ void KateBuffer::doHighlight (int startLine, int endLine, bool invalidate)
 
     // move around the lines
     prevLine = textLine;
-    textLine = nextLine;
   }
 
   /**
@@ -510,6 +740,9 @@ void KateBuffer::doHighlight (int startLine, int endLine, bool invalidate)
                              qMin(lines()-1, (last_line_spellchecking==-1)?qMax (current_line, oldHighlighted):last_line_spellchecking));
     }
   }
+  // emit that we have changed the folding
+  if (codeFoldingUpdate)
+    emit codeFoldingUpdated();
 
 #ifdef BUFFER_DEBUGGING
   kDebug (13020) << "HIGHLIGHTED END --- NEED HL, LINESTART: " << startLine << " LINEEND: " << endLine;
@@ -519,221 +752,15 @@ void KateBuffer::doHighlight (int startLine, int endLine, bool invalidate)
 #endif
 }
 
-KTextEditor::Range KateBuffer::computeFoldingRangeForStartLine (int startLine)
-{
-  /**
-   * ensure valid input
-   */
-  Q_ASSERT (startLine >= 0);
-  Q_ASSERT (startLine < lines());
-  
-  /**
-   * no highlighting, no folding, ATM
-   */
-  if (!m_highlight || m_highlight->noHighlighting())
-    return KTextEditor::Range::invalid();
-  
-  /**
-   * first: get the wanted start line highlighted
-   */
-  ensureHighlighted (startLine);
-  Kate::TextLine startTextLine = plainLine (startLine);
-  
-  /**
-   * return if no folding start!
-   */
-  if (!startTextLine->markedAsFoldingStart ())
-    return KTextEditor::Range::invalid(); 
-  
-  /**
-   * now: decided if indentation based folding or not!
-   */
-  if (startTextLine->markedAsFoldingStartIndentation ()) {
-    /**
-     * get our start indentation level
-     */
-    const int startIndentation = startTextLine->indentDepth (tabWidth());
-    
-    /**
-     * search next line with indentation level <= our one
-     */
-    int lastLine = startLine + 1;
-    for (; lastLine < lines(); ++lastLine) {
-      /**
-       * get line
-       */
-      Kate::TextLine textLine = plainLine (lastLine);
-  
-      /**
-       * indentation higher than our start line? continue
-       */
-      if (startIndentation < textLine->indentDepth (tabWidth()))
-        continue;
-      
-      /**
-       * empty line? continue
-       */
-      if (m_highlight->isEmptyLine (textLine.data()))
-        continue;
-      
-      /**
-       * else, break
-       */
-      break;
-    }
-    
-    /**
-     * lastLine is always one too much
-     */
-    --lastLine;
-    
-    /**
-     * backtrack all empty lines, we don't want to add them to the fold!
-     */
-    while (lastLine > startLine) {
-      if (m_highlight->isEmptyLine (plainLine (lastLine).data()))
-        --lastLine;
-      else
-        break;
-    }
-    
-    /**
-     * we shall not fold one-liners
-     */
-    if (lastLine == startLine)
-      return KTextEditor::Range::invalid(); 
-    
-    /**
-     * be done now
-     */
-    return KTextEditor::Range (KTextEditor::Cursor (startLine, 0), KTextEditor::Cursor (lastLine, plainLine (lastLine)->length()));
+void KateBuffer::codeFoldingColumnUpdate(int lineNr) {
+  Kate::TextLine line=plainLine(lineNr);
+  if (!line) return;
+  if (line->foldingColumnsOutdated()) {
+    line->setFoldingColumnsOutdated(false);
+    bool tmp;
+    QVector<int> folding=line->foldingListArray();
+    m_regionTree.updateLine(lineNr,folding,&tmp,true,false);
   }
-  
-  /**
-   * 'normal' attribute based folding, aka token based like '{' BLUB '}'
-   */
-  
-  /**
-   * first step: search the first region type, that stays open for the start line
-   */
-  short openedRegionType = 0;
-  int openedRegionOffset = -1;
-  {
-    /**
-     * mapping of type to "first" offset of it and current number of not matched openings
-     */
-    QHash<short, QPair<int, int> > foldingStartToOffsetAndCount;
-    
-    /**
-     * walk over all attributes of the line and compute the matchings
-     */
-    const QVector<Kate::TextLineData::Attribute> &startLineAttributes = startTextLine->attributesList();
-    for ( int i = 0; i < startLineAttributes.size(); ++i ) {
-      /**
-       * folding close?
-       */
-      if (startLineAttributes[i].foldingValue < 0) {
-        /**
-         * search for this type, try to decrement counter, perhaps erase element!
-         */
-        QHash<short, QPair<int, int> >::iterator end = foldingStartToOffsetAndCount.find (-startLineAttributes[i].foldingValue);
-        if (end != foldingStartToOffsetAndCount.end()) {
-          if (end.value().second > 1)
-            --(end.value().second);
-          else
-            foldingStartToOffsetAndCount.erase (end);
-        }
-      }
-      
-      /**
-       * folding open?
-       */
-      if (startLineAttributes[i].foldingValue > 0) {
-        /**
-         * search for this type, either insert it, with current offset or increment counter!
-         */
-        QHash<short, QPair<int, int> >::iterator start = foldingStartToOffsetAndCount.find (startLineAttributes[i].foldingValue);
-        if (start != foldingStartToOffsetAndCount.end())
-          ++(start.value().second);
-        else
-          foldingStartToOffsetAndCount.insert (startLineAttributes[i].foldingValue, qMakePair (startLineAttributes[i].offset, 1));
-      }
-    }
-    
-    /**
-     * compute first type with offset
-     */
-    QHashIterator<short, QPair<int, int> > hashIt (foldingStartToOffsetAndCount);
-    while (hashIt.hasNext()) {
-       hashIt.next();
-       if (openedRegionOffset == -1 || hashIt.value().first < openedRegionOffset) {
-          openedRegionType = hashIt.key();
-          openedRegionOffset = hashIt.value().first;
-       }
-    }
-  }
-  
-  /**
-   * no opening region found, bad, nothing to do
-   */
-  if (openedRegionType == 0)
-    return KTextEditor::Range::invalid();
-
-  /**
-   * second step: search for matching end region marker!
-   */
-  int countOfOpenRegions = 1;
-  for (int line = startLine + 1; line < lines(); ++line) {
-    /**
-     * ensure line is highlighted
-     */
-    ensureHighlighted (line);
-    Kate::TextLine textLine = plainLine (line);
-    
-    /**
-     * search for matching end marker
-     */
-    const QVector<Kate::TextLineData::Attribute> &lineAttributes = textLine->attributesList();
-    for (int i = 0; i < lineAttributes.size(); ++i) {
-      /**
-       * matching folding close?
-       */
-      if (lineAttributes[i].foldingValue == -openedRegionType) {
-        --countOfOpenRegions;
-        
-        /**
-         * end reached?
-         * compute resulting range!
-         */
-        if (countOfOpenRegions == 0) {
-          /**
-           * special handling of end: if end is at column 0 of a line, move it to end of previous line!
-           * fixes folding for stuff like
-           * #pragma mark END_OLD_AND_START_NEW_REGION
-           */
-          KTextEditor::Cursor endCursor (line, lineAttributes[i].offset);
-          if (endCursor.column() == 0 && endCursor.line() > 0)
-            endCursor = KTextEditor::Cursor (endCursor.line()-1, plainLine (lines()-1)->length());
-          
-          /**
-           * return computed range
-           */
-          return KTextEditor::Range (KTextEditor::Cursor (startLine, openedRegionOffset), endCursor);
-        }
-      }
-    
-      /**
-       * matching folding open?
-       */
-      if (lineAttributes[i].foldingValue == openedRegionType)
-        ++countOfOpenRegions;
-    }
-  }
-  
-  /**
-   * if we arrive here, the opened range spans to the end of the document!
-   */
-  return KTextEditor::Range (KTextEditor::Cursor (startLine, openedRegionOffset), KTextEditor::Cursor (lines()-1, plainLine (lines()-1)->length()));
 }
 
 // kate: space-indent on; indent-width 2; replace-tabs on;
