@@ -168,9 +168,9 @@ void KateLayoutCache::updateViewCache(const KTextEditor::Cursor& startPos, int n
 
   int realLine;
   if (newViewLineCount == -1)
-    realLine = m_renderer->folding().visibleLineToLine(m_renderer->folding().lineToVisibleLine(startPos.line()));
+    realLine = m_renderer->doc()->getRealLine(m_renderer->doc()->getVirtualLine(startPos.line()));
   else
-    realLine = m_renderer->folding().visibleLineToLine(startPos.line());
+    realLine = m_renderer->doc()->getRealLine(startPos.line());
   int _viewLine = 0;
 
   if (wrap()) {
@@ -264,7 +264,7 @@ void KateLayoutCache::updateViewCache(const KTextEditor::Cursor& startPos, int n
 
     if (_viewLine > l->viewLineCount() - 1) {
       int virtualLine = l->virtualLine() + 1;
-      realLine = m_renderer->folding().visibleLineToLine(virtualLine);
+      realLine = m_renderer->doc()->getRealLine(virtualLine);
       _viewLine = 0;
       if (realLine < m_renderer->doc()->lines()) {
         l = line(realLine, virtualLine);
@@ -281,10 +281,6 @@ KateLineLayoutPtr KateLayoutCache::line( int realLine, int virtualLine )
 {
   if (m_lineLayouts.contains(realLine)) {
     KateLineLayoutPtr l = m_lineLayouts[realLine];
-    
-    // ensure line is OK
-    Q_ASSERT (l->line() == realLine);
-    Q_ASSERT (realLine < m_renderer->doc()->buffer().lines());
 
     if (virtualLine != -1)
       l->setVirtualLine(virtualLine);
@@ -311,7 +307,7 @@ KateLineLayoutPtr KateLayoutCache::line( int realLine, int virtualLine )
   if (realLine < 0 || realLine >= m_renderer->doc()->lines())
     return KateLineLayoutPtr();
 
-  KateLineLayoutPtr l(new KateLineLayout(*m_renderer));
+  KateLineLayoutPtr l(new KateLineLayout(m_renderer->doc()));
   l->setLine(realLine, virtualLine);
 
   // Mark it dirty, because it may not have the syntax highlighting applied
@@ -390,7 +386,6 @@ int KateLayoutCache::viewLine(const KTextEditor::Cursor& realCursor)
 {
   if (realCursor.column() <= 0 || realCursor.line() < 0) return 0;
 
-  Q_ASSERT(realCursor.line() < m_renderer->doc()->lines());
   KateLineLayoutPtr thisLine = line(realCursor.line());
 
   for (int i = 0; i < thisLine->viewLineCount(); ++i) {
@@ -404,14 +399,8 @@ int KateLayoutCache::viewLine(const KTextEditor::Cursor& realCursor)
 
 int KateLayoutCache::displayViewLine(const KTextEditor::Cursor& virtualCursor, bool limitToVisible)
 {
-  if (!virtualCursor.isValid())
-    return -1;
-
   KTextEditor::Cursor work = viewCacheStart();
-  
-  // only try this with valid lines!
-  if (work.isValid())
-    work.setLine(m_renderer->folding().lineToVisibleLine(work.line()));
+  work.setLine(m_renderer->doc()->getVirtualLine(work.line()));
 
   if (!work.isValid())
     return virtualCursor.line();
@@ -437,7 +426,7 @@ int KateLayoutCache::displayViewLine(const KTextEditor::Cursor& virtualCursor, b
   // FIXME switch to using ranges? faster?
   if (forwards) {
     while (work.line() != virtualCursor.line()) {
-      ret += viewLineCount(m_renderer->folding().visibleLineToLine(work.line()));
+      ret += viewLineCount(m_renderer->doc()->getRealLine(work.line()));
       work.setLine(work.line() + 1);
       if (limitToVisible && ret > limit)
         return -1;
@@ -445,7 +434,7 @@ int KateLayoutCache::displayViewLine(const KTextEditor::Cursor& virtualCursor, b
   } else {
     while (work.line() != virtualCursor.line()) {
       work.setLine(work.line() - 1);
-      ret -= viewLineCount(m_renderer->folding().visibleLineToLine(work.line()));
+      ret -= viewLineCount(m_renderer->doc()->getRealLine(work.line()));
       if (limitToVisible && ret < 0)
         return -1;
     }
@@ -453,7 +442,7 @@ int KateLayoutCache::displayViewLine(const KTextEditor::Cursor& virtualCursor, b
 
   // final difference
   KTextEditor::Cursor realCursor = virtualCursor;
-  realCursor.setLine(m_renderer->folding().visibleLineToLine(realCursor.line()));
+  realCursor.setLine(m_renderer->doc()->getRealLine(realCursor.line()));
   if (realCursor.column() == -1) realCursor.setColumn(m_renderer->doc()->lineLength(realCursor.line()));
   ret += viewLine(realCursor);
 
@@ -481,17 +470,11 @@ void KateLayoutCache::viewCacheDebugOutput( ) const
 {
   kDebug( 13033 ) << "Printing values for " << m_textLayouts.count() << " lines:";
   if (m_textLayouts.count())
-  {
   foreach (const KateTextLayout& t, m_textLayouts)
     if (t.isValid())
-    {
       t.debugOutput();
-    }
     else
-    {
       kDebug( 13033 ) << "Line Invalid.";
-    }
-  }
 }
 
 void KateLayoutCache::wrapLine (const KTextEditor::Cursor &position)
