@@ -30,7 +30,6 @@
 #include "kateconfig.h"
 #include "kateglobal.h"
 #include "katerenderer.h"
-#include "katebuffer.h"
 #include "kateview.h"
 #include "spellcheck.h"
 #include "spellingmenu.h"
@@ -58,7 +57,7 @@ KateOnTheFlyChecker::KateOnTheFlyChecker(KateDocument *document)
           this, SLOT(addView(KTextEditor::Document*,KTextEditor::View*)));
   connect(document, SIGNAL(highlightingModeChanged(KTextEditor::Document*)),
           this, SLOT(updateConfig()));
-  connect(&document->buffer(), SIGNAL(respellCheckBlock(KateDocument*,int,int)),
+  connect(document, SIGNAL(respellCheckBlock(KateDocument*,int,int)),
           this, SLOT(handleRespellCheckBlock(KateDocument*,int,int)));
 
   // load the settings for the speller
@@ -412,7 +411,10 @@ void KateOnTheFlyChecker::performSpellCheck()
             SLOT(misspelling(QString,int)));
     connect(m_backgroundChecker, SIGNAL(done()), this, SLOT(spellCheckDone()));
 
+#if KDE_IS_VERSION(4,5,2)
+// guard necessary to ensure compilation of KatePart's git repository against <= 4.5.1
     m_backgroundChecker->restore(KGlobal::config().data());
+#endif
   }
   m_backgroundChecker->setSpeller(m_speller);
   m_backgroundChecker->setText(text); // don't call 'start()' after this!
@@ -672,9 +674,12 @@ void KateOnTheFlyChecker::updateConfig()
   ON_THE_FLY_DEBUG;
   m_speller.restore(KGlobal::config().data());
 
+#if KDE_IS_VERSION(4,5,2)
+// guard necessary to ensure compilation of KatePart's git repository against <= 4.5.1
   if(m_backgroundChecker) {
     m_backgroundChecker->restore(KGlobal::config().data());
   }
+#endif
 }
 
 void KateOnTheFlyChecker::refreshSpellCheck(const KTextEditor::Range &range)
