@@ -372,7 +372,7 @@ void KateCodeFoldingNode::updateSelf()
       KateCodeFoldingNode* child = startChildAt(i);
 
       // If "child" is lower than parent's pair, this node is not it's child anymore.
-      if (child->m_position > matchingNode()->m_position) {
+      if (child->m_position >= matchingNode()->m_position) {
         removeStart(child);
         QVector <KateCodeFoldingNode *> temptExcessList;
 
@@ -524,13 +524,19 @@ void KateCodeFoldingTree::collapseAll_dsComments()
 {
   if (m_root->noStartChildren())
     return;
-  
+
+  bool foldingChanged = false;
   foreach (KateCodeFoldingNode *child, m_root->m_startChildren) {
     bool isComment = m_buffer->document()->isComment(child->getLine(),child->getColumn());
     if (!m_hiddenNodes.contains(child) && isComment) {
       foldNode(child);
+      foldingChanged = true;
     }
   }
+
+  if (foldingChanged) {
+    emit regionVisibilityChanged();
+  }
 }
 
 void KateCodeFoldingTree::collapseLevel(int level, KateCodeFoldingNode *node, int nodeLevel)
@@ -541,6 +547,7 @@ void KateCodeFoldingTree::collapseLevel(int level, KateCodeFoldingNode *node, in
   // if "level" was reached, we fold the node
   if (level == nodeLevel) {
     foldNode(node);
+    emit regionVisibilityChanged();
     return;
   }
 
@@ -561,6 +568,7 @@ int KateCodeFoldingTree::collapseOne(int realLine, int column)
     return 0;
 
   foldNode(nodeToFold);
+  emit regionVisibilityChanged();
 
   return 0;
 }
@@ -571,11 +579,17 @@ void KateCodeFoldingTree::collapseToplevelNodes()
   if (m_root->noStartChildren())
     return;
 
+  bool changed = false;
   foreach (KateCodeFoldingNode *child, m_root->m_startChildren) {
     if (!m_hiddenNodes.contains(child)) {
       foldNode(child);
+      changed = true;
     }
   }
+
+  if (changed) {
+    emit regionVisibilityChanged();
+  }
 }
 
 // This method removes a node from the line map structure
@@ -598,6 +612,8 @@ void KateCodeFoldingTree::deleteNodeFromMap(KateCodeFoldingNode *node)
 
   if (m_hiddenNodes.contains(node)) {
     unfoldNode(node);
+    // do not emit regionVisibilityChanged(); as it leads to double deletion
+    // of folding nodes
   }
 }
 
@@ -650,6 +666,7 @@ void KateCodeFoldingTree::ensureVisible(int l)
       matchNode = m_rootMatch;
     if (node->getLine() < l && l <= matchNode->getLine()) {
       unfoldNode(node);
+      emit regionVisibilityChanged();
       break;
     }
   }
@@ -686,6 +703,7 @@ void KateCodeFoldingTree::expandLevel(int level, KateCodeFoldingNode *node, int
   if (level == nodeLevel) {
     if (!node->isVisible()) {
       unfoldNode(node);
+      emit regionVisibilityChanged();
     }
     return;
   }
@@ -704,6 +722,7 @@ void KateCodeFoldingTree::expandOne(int realLine, int column)
     return;
 
   unfoldNode(nodeToUnfold);
+  emit regionVisibilityChanged();
 }
 
 // This method unfold the top level (depth(node = 1)) nodes
@@ -712,11 +731,16 @@ void KateCodeFoldingTree::expandToplevelNodes()
   if (m_root->noStartChildren())
     return;
 
+  bool changed = false;
   foreach (KateCodeFoldingNode *child, m_root->m_startChildren) {
     if (m_hiddenNodes.contains(child)) {
       unfoldNode(child);
+      changed = true;
     }
   }
+  if (changed) {
+    emit regionVisibilityChanged();
+  }
 }
 
 // Searches for the first start node above
@@ -1353,8 +1377,6 @@ void KateCodeFoldingTree::foldNode(KateCodeFoldingNode *node)
 
   oldHiddenNodes.clear();
 
-  emit regionVisibilityChanged ();
-
   //printMapping();
 }
 
@@ -1404,8 +1426,6 @@ void KateCodeFoldingTree::unfoldNode(KateCodeFoldingNode *node)
   }
 
   replaceFoldedNodeWithList(node, newFoldedNodes);
-
-  emit regionVisibilityChanged();
 }
 
 // The method replace a single node with a list of nodes into the m_hiddenNodes data strucutre
@@ -1448,7 +1468,7 @@ void KateCodeFoldingTree::toggleRegionVisibility(int l)
   if (!m_lineMapping.contains(l))
     return;
 
-  bool foldedFound = false;
+  bool foldChanged = false;
 
   foreach (KateCodeFoldingNode* node, m_lineMapping.value(l)) {
 
@@ -1459,12 +1479,12 @@ void KateCodeFoldingTree::toggleRegionVisibility(int l)
     // If there are any folded nodes, they have priority
     if (!node->isVisible()) {
       unfoldNode(node);
-      foldedFound = true;
+      foldChanged = true;
     }
   }
 
   // If there were no folded nodes, we'll folde the first node
-  if (!foldedFound) {
+  if (!foldChanged) {
     foreach (KateCodeFoldingNode* node, m_lineMapping.value(l)) {
 
       // We can fold/unfold only start nodes
@@ -1474,10 +1494,15 @@ void KateCodeFoldingTree::toggleRegionVisibility(int l)
       // We fold the first unfolded node and exit
       if (node->isVisible()) {
         foldNode(node);
+        foldChanged = true;
         break;
       }
     }
   }
+
+  if (foldChanged) {
+    emit regionVisibilityChanged();
+  }
 }
 
 // changed = true, if there is there is a new node on the line / a node was deleted from the line
@@ -1547,7 +1572,7 @@ void KateCodeFoldingTree::updateMapping(int line, const QVector<int> &newColumns
     // then we update the column (maybe the column has changed).
     else if (oldLineMapping[index_old]->m_type == newColumns[index_new - 1]) {
       // I can simply change the column only if this change will not mess the order of the line
-      if ((index_old == (oldLineMapping.size() - 1)) || oldLineMapping[index_old + 1]->getColumn() >= newColumns[index_new])
+      if ((index_old == (oldLineMapping.size() - 1)) || oldLineMapping[index_old + 1]->getColumn() > newColumns[index_new])
       {
         if (newColumns[index_new - 1] < -1) {
           oldLineMapping[index_old]->setColumn(newColumns[index_new] - 1);
@@ -1633,10 +1658,15 @@ void KateCodeFoldingTree::applyFoldingState()
     int line = itLines.next();
     int column = itColumns.next();
 
-    Q_ASSERT(m_lineMapping.contains(line));
+    if (!m_lineMapping.contains(line)) {
+      // if the node does not exist, abort restoring the folding state, as
+      // the file probably changed, hence, folds would be wrong anyways
+      return;
+    }
     foreach (KateCodeFoldingNode* node, m_lineMapping.value(line)) {
       if (node->getColumn() == column) {
         foldNode(node);
+        emit regionVisibilityChanged();
         break;
       }
     }
