@@ -215,7 +215,7 @@ int KateHlWordDetect::checkHgl(const QString& text, int offset, int len)
   //NOTE: word boundary means: any non-word character.
 
   // make sure there is no letter or number before the word starts
-  if (offset > 0 && isWordCharacter(text.at(offset - 1))) {
+  if (offset > 0 && isWordCharacter(text.at(offset))) {
     return 0;
   }
   offset = KateHlStringDetect::checkHgl(text, offset, len);
@@ -284,19 +284,8 @@ KateHlKeyword::KateHlKeyword (int attribute, KateHlContextModification context,
 
 KateHlKeyword::~KateHlKeyword ()
 {
-  qDeleteAll(dict);
-}
-
-QSet<QString> KateHlKeyword::allKeywords() const
-{
-  QSet<QString> result;
-  foreach ( const QSet<QString>* v, dict ) {
-    if ( ! v ) {
-      continue;
-    }
-    result.unite(*v);
-  }
-  return result;
+  for (int i=0; i < dict.size(); ++i)
+    delete dict[i];
 }
 
 void KateHlKeyword::addList(const QStringList& list)
@@ -693,18 +682,16 @@ KateHlItem *KateHlRegExpr::clone(const QStringList *args)
   ret->dynamicChild = true;
   return ret;
 }
-// //END
+//END
 
 //BEGIN KateHlLineContinue
-KateHlLineContinue::KateHlLineContinue(int attribute, KateHlContextModification context, signed char regionId, signed char regionId2, QChar c)
-  : KateHlItem(attribute, context, regionId, regionId2)
-  , m_trailer(c.isNull() ? QLatin1Char('\\') : c)
-{
+KateHlLineContinue::KateHlLineContinue(int attribute, KateHlContextModification context, signed char regionId,signed char regionId2)
+  : KateHlItem(attribute,context,regionId,regionId2) {
 }
 
 int KateHlLineContinue::checkHgl(const QString& text, int offset, int len)
 {
-  if ((len == 1) && (text[offset] == m_trailer))
+  if ((len == 1) && (text[offset] == '\\'))
     return ++offset;
 
   return 0;
@@ -838,29 +825,25 @@ KateHl2CharDetect::KateHl2CharDetect(int attribute, KateHlContextModification co
 //END KateHl2CharDetect
 
 //BEGIN KateHlContext
-KateHlContext::KateHlContext (const QString &_hlId, int attribute, KateHlContextModification _lineEndContext, bool _fallthrough,
-	KateHlContextModification _fallthroughContext, bool _dynamic, bool _noIndentationBasedFolding,
-                  bool _emptyLineContext, KateHlContextModification _emptyLineContextModification)
+KateHlContext::KateHlContext (const QString &_hlId, int attribute, KateHlContextModification _lineEndContext, KateHlContextModification _lineBeginContext, bool _fallthrough,
+	KateHlContextModification _fallthroughContext, bool _dynamic, bool _noIndentationBasedFolding)
 {
   hlId = _hlId;
   attr = attribute;
   lineEndContext = _lineEndContext;
+  lineBeginContext = _lineBeginContext;
   fallthrough = _fallthrough;
   ftctx = _fallthroughContext;
   dynamic = _dynamic;
   dynamicChild = false;
   noIndentationBasedFolding=_noIndentationBasedFolding;
-  emptyLineContext = _emptyLineContext;
-  emptyLineContextModification = _emptyLineContextModification;
   if (_noIndentationBasedFolding) kDebug(13010)<<QString("**********************_noIndentationBasedFolding is TRUE*****************");
 
 }
 
 KateHlContext *KateHlContext::clone(const QStringList *args)
 {
-  KateHlContext *ret = new KateHlContext(hlId, attr, lineEndContext, fallthrough, ftctx, false,noIndentationBasedFolding
-      , emptyLineContext, emptyLineContextModification
-  );
+  KateHlContext *ret = new KateHlContext(hlId, attr, lineEndContext, lineBeginContext, fallthrough, ftctx, false,noIndentationBasedFolding);
 
   for (int n=0; n < items.size(); ++n)
   {
