@@ -31,8 +31,6 @@
 #include "kateschema.h"
 #include "katetextline.h"
 #include "kateview.h"
-#include "katebuffer.h"
-#include "katetextfolding.h"
 
 #include <kapplication.h>
 #include <kcolorbutton.h>
@@ -44,10 +42,10 @@
 #include <kuser.h> // for loginName
 #include <klineedit.h>
 #include <knuminput.h>
-#include <kcombobox.h>
 
 #include <QtGui/QPainter>
 #include <QtGui/QCheckBox>
+#include <QtGui/QComboBox>
 #include <QtGui/QGroupBox>
 #include <QtGui/QPrintDialog>
 #include <QtGui/QPrinter>
@@ -132,8 +130,7 @@ bool KatePrinter::print (KateDocument *doc)
   {
     writeSettings(printer);
 
-    Kate::TextFolding folding (doc->buffer());
-    KateRenderer renderer(doc, folding, doc->activeKateView());
+    KateRenderer renderer(doc, doc->activeKateView());
     renderer.config()->setSchema (kpl->colorScheme());
     renderer.setPrinterFriendly(true);
 
@@ -151,6 +148,8 @@ bool KatePrinter::print (KateDocument *doc)
     uint lineCount = 0;
     uint maxWidth = pdmWidth;
     int headerWidth = pdmWidth;
+    int startCol = 0;
+    int endCol = 0;
     bool pageStarted = true;
     int remainder = 0; // remaining sublines from a wrapped line (for the top of a new page)
 
@@ -241,7 +240,7 @@ bool KatePrinter::print (KateDocument *doc)
         tags["y"] =  KGlobal::locale()->formatDate(dt.date(), KLocale::ShortDate);
         tags["Y"] =  KGlobal::locale()->formatDate(dt.date(), KLocale::LongDate);
         tags["f"] =  doc->url().fileName();
-        tags["U"] =  doc->url().pathOrUrl();
+        tags["U"] =  doc->url().prettyUrl();
         if ( selectionOnly )
         {
           QString s( i18n("(Selection of) ") );
@@ -357,8 +356,8 @@ bool KatePrinter::print (KateDocument *doc)
         // calculate total layouted lines in the document
         int totalLines = 0;
         // TODO: right now ignores selection printing
-        for (unsigned int i = firstline; i <= lastline; ++i) {
-          KateLineLayoutPtr rangeptr(new KateLineLayout(renderer));
+        for (int i = firstline; i <= lastline; ++i) {
+          KateLineLayoutPtr rangeptr(new KateLineLayout(doc));
           rangeptr->setLine(i);
           renderer.layoutLine(rangeptr, (int)maxWidth, false);
           totalLines += rangeptr->viewLineCount();
@@ -387,6 +386,9 @@ bool KatePrinter::print (KateDocument *doc)
      */
     while (  lineCount <= lastline  )
     {
+      startCol = 0;
+      endCol = 0;
+
       if ( y + fontHeight > maxHeight )
       {
         kDebug(13020)<<"Starting new page,"<<lineCount<<"lines up to now.";
@@ -607,7 +609,7 @@ bool KatePrinter::print (KateDocument *doc)
 
       // HA! this is where we print [part of] a line ;]]
       // FIXME Convert this function + related functionality to a separate KatePrintView
-      KateLineLayoutPtr rangeptr(new KateLineLayout(renderer));
+      KateLineLayoutPtr rangeptr(new KateLineLayout(doc));
       rangeptr->setLine(lineCount);
       renderer.layoutLine(rangeptr, (int)maxWidth, false);
 
@@ -1161,12 +1163,8 @@ KatePrintLayout::KatePrintLayout( QWidget *parent)
   // set defaults:
   sbBoxMargin->setValue( 6 );
   gbBoxProps->setEnabled( false );
-  
-  Q_FOREACH (const KateSchema &schema, KateGlobal::self()->schemaManager()->list())
-    cmbSchema->addItem (schema.translatedName(), QVariant (schema.rawName));
-    
-  // default is printing, MUST BE THERE
-  cmbSchema->setCurrentIndex (cmbSchema->findData (QVariant("Printing")));
+  cmbSchema->addItems (KateGlobal::self()->schemaManager()->list ());
+  cmbSchema->setCurrentIndex( 1 );
 
   // whatsthis
   cmbSchema->setWhatsThis(i18n(
@@ -1195,7 +1193,7 @@ KatePrintLayout::~KatePrintLayout()
 
 QString KatePrintLayout::colorScheme()
 {
-  return cmbSchema->itemData(cmbSchema->currentIndex()).toString();
+  return cmbSchema->currentText();
 }
 
 bool KatePrintLayout::useBackground()
@@ -1230,11 +1228,8 @@ void KatePrintLayout::readSettings()
   
   KConfigGroup layoutGroup(&printGroup, "Layout");
 
-  // get color schema back
-  QString colorScheme = layoutGroup.readEntry( "ColorScheme", "Printing" );
-  int index = cmbSchema->findData (QVariant (colorScheme));
-  if (index != -1)
-    cmbSchema->setCurrentIndex ( index );
+  QString colorScheme = layoutGroup.readEntry( "ColorScheme", KateGlobal::self()->schemaManager()->name(1) );
+  cmbSchema->setCurrentItem( colorScheme );
 
   bool isBackgroundChecked = layoutGroup.readEntry( "BackgroundColorEnabled", false );
   cbDrawBackground->setChecked( isBackgroundChecked );
